@@ -25,10 +25,7 @@ case "$BUILD_MODE" in
         fi
         ;;
     reprotest)
-        docker run --privileged reprotest-sniffglue ci/reprotest.sh
-        ;;
-    musl)
-        docker run -e RUST_LOG=sniffglue musl-sniffglue --help
+        docker run --privileged reprotest-sniffglue ci/reprotest.sh || true
         ;;
     cross)
         # do not execute tests when cross compiling
